@@ -27,23 +27,23 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
         extinction_json = json.load(f)
         
     ## Modify the dictionary values to add new entries
-    dict_Materials["ZrO2_mono"] = ["ZrO2_mono", [5.1471,  5.2125, 5.3129, 90, 99.23, 90], "14"]
-    dict_Materials["ZrO2_tet"] = ["ZrO2_tet", [3.64,  3.64, 5.27, 90, 90, 90], "137"]
+    dict_Materials["ZrO2_mono"] = ["ZrO2_mono", [5.1471,  5.2125, 5.3129, 90, 99.23, 90], "VO2_mono"]
+    dict_Materials["ZrO2_tet"] = ["ZrO2_tet", [3.64,  3.64, 5.27, 90, 90, 90], "VO2_mono2tet"]
     dict_Materials["ZrO2_cub"] = ["ZrO2_cub", [4.625,  4.625, 4.625, 90, 90, 90], "225"]
-    dict_Materials["Zr_alpha"] = ["Zr_alpha", [3.23,  3.23, 5.15, 90, 90, 120], "194"]
+    dict_Materials["Zr_alpha"] = ["Zr_alpha", [3.23,  3.23, 5.15, 90, 90, 120], "hcp"]
     dict_Materials["Zr_beta"] = ["Zr_beta", [3.62,  3.62, 3.62, 90, 90, 90], "229"]
     dict_Materials["Nb_beta"] = ["Nb_beta", [3.585,  3.585, 3.585, 90, 90, 90], "229"]
-    dict_Materials["Zr_Nb_Fe"] = ["Zr_Nb_Fe", [4.879,  4.879, 7.992, 90, 90, 120], "194"]
+    dict_Materials["Zr_Nb_Fe"] = ["Zr_Nb_Fe", [4.879,  4.879, 7.992, 90, 90, 120], "hcp"]
     dict_Materials["Cr"] = ["Cr", [2.87,  2.87, 2.87, 90, 90, 90], "229"]
-    dict_Materials["C14"] = ["C14", [5.05,  5.05, 8.24, 90, 90, 120], "194"]
+    dict_Materials["C14"] = ["C14", [5.05,  5.05, 8.24, 90, 90, 120], "hcp"]
     dict_Materials["C15"] = ["C15", [7.15,  7.15, 7.15, 90, 90, 90], "227"]
     
-    extinction_json["14"] = "14"
-    extinction_json["137"] = "137"
-    extinction_json["225"] = "225"
-    extinction_json["194"] = "194"
+    extinction_json["VO2_mono2tet"] = "VO2_mono2tet"
+    extinction_json["VO2_mono"] = "VO2_mono"
+    extinction_json["hcp"] = "hcp"
     extinction_json["229"] = "229"
     extinction_json["227"] = "227"
+    extinction_json["225"] = "225"
     
     ## dump the json back with new values
     with open(r'C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\lauetools\material.json', 'w') as fp:
@@ -51,224 +51,7 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
 
     with open(r'C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\lauetools\extinction.json', 'w') as fp:
         json.dump(extinction_json, fp)
-    ### Configuration for Guillou experiments
-    ZrCr_irrad_wo_Nb = {
-                        # =============================================================================
-                        #       GENERATION OF DATASET              
-                        # =============================================================================
-                        "material_": ["Zr_alpha",
-                                      "Cr",
-                                      "C14",
-                                      "C15",
-                                      ],             ## same key as used in dict_LaueTools
-                        "symmetry": ["hexagonal",
-                                     "cubic",
-                                     "hexagonal",
-                                     "cubic",
-                                     ],           ## crystal symmetry of material_
-                        "SG": [194,
-                               229,
-                               194,
-                               227,
-                               ],                     ## Space group of material_ (None if not known)
-                        "hkl_max_identify" : [6,
-                                              5,
-                                              6,
-                                              5
-                                              ],        ## Maximum hkl index to classify in a Laue pattern
-                        "nb_grains_per_lp" : [1,
-                                              1,
-                                              1,
-                                              1
-                                              ],        ## max grains to be generated in a Laue Image
-                        "grains_nb_simulate" : 500,    ## Number of orientations to generate (takes advantage of crystal symmetry)
-                        # =============================================================================
-                        #       Training paarmeters             
-                        # =============================================================================
-                        "freq_rmv_classhkl" : [100,
-                                               100,
-                                               100,
-                                               100
-                                               ],
-                        "keep_length_classhkl" : [25,
-                                                  10,
-                                                  25,
-                                                  10
-                                                  ],}
-    
-    ZrCr_w_wo_Nb = {
-                        # =============================================================================
-                        #       GENERATION OF DATASET              
-                        # =============================================================================
-                        "material_": ["Zr_alpha",
-                                      "Cr",
-                                      "Zr_Nb_Fe",
-                                      ],             ## same key as used in dict_LaueTools
-                        "symmetry": ["hexagonal",
-                                     "cubic",
-                                     "hexagonal",
-                                     ],           ## crystal symmetry of material_
-                        "SG": [194,
-                               229,
-                               194,
-                               ],                     ## Space group of material_ (None if not known)
-                        "hkl_max_identify" : [6,
-                                              5,
-                                              6,
-                                              ],        ## Maximum hkl index to classify in a Laue pattern
-                        "nb_grains_per_lp" : [1,
-                                              1,
-                                              1,
-                                              ],        ## max grains to be generated in a Laue Image
-                        "grains_nb_simulate" : 500,    ## Number of orientations to generate (takes advantage of crystal symmetry)
-                        # =============================================================================
-                        #       Training paarmeters             
-                        # =============================================================================
-                        "freq_rmv_classhkl" : [100,
-                                               100,
-                                               100,
-                                               ],
-                        "keep_length_classhkl" : [25,
-                                                  10,
-                                                  25,
-                                                  ],}
-    
-    oxidealloy_w_wo_Nb = {
-                        # =============================================================================
-                        #       GENERATION OF DATASET              
-                        # =============================================================================
-                        "material_": ["Zr_alpha",
-                                      "ZrO2_mono",
-                                      "ZrO2_tet",
-                                      "ZrO2_cub",
-                                      "Zr_Nb_Fe",
-                                      ],             ## same key as used in dict_LaueTools
-                        "symmetry": ["hexagonal",
-                                     "monoclinic",
-                                     "tetragonal",
-                                     "cubic",
-                                     "hexagonal",
-                                     ],           ## crystal symmetry of material_
-                        "SG": [194,
-                               14,
-                               137,
-                               225,
-                               194,
-                               ],                     ## Space group of material_ (None if not known)
-                        "hkl_max_identify" : [5,
-                                              6,
-                                              5,
-                                              5,
-                                              5,
-                                              ],        ## Maximum hkl index to classify in a Laue pattern
-                        "nb_grains_per_lp" : [1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              ],        ## max grains to be generated in a Laue Image
-                        "grains_nb_simulate" : 500,    ## Number of orientations to generate (takes advantage of crystal symmetry)
-                        # =============================================================================
-                        #       Training paarmeters             
-                        # =============================================================================
-                        "freq_rmv_classhkl" : [100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               ],
-                        "keep_length_classhkl" : [25,
-                                                  100,
-                                                  60,
-                                                  10,
-                                                  25,
-                                                  ],}
-    
-    all_guillou = {
-                        # =============================================================================
-                        #       GENERATION OF DATASET              
-                        # =============================================================================
-                        "material_": ["Zr_alpha",
-                                      "ZrO2_mono",
-                                      "ZrO2_tet",
-                                      "ZrO2_cub",
-                                      "Zr_Nb_Fe",
-                                      "Cr",
-                                      "C14",
-                                      "C15",
-                                      "Zr_beta",
-                                      "Nb_beta"
-                                      ],             ## same key as used in dict_LaueTools
-                        "symmetry": ["hexagonal",
-                                     "monoclinic",
-                                     "tetragonal",
-                                     "cubic",
-                                     "hexagonal",
-                                     "cubic",
-                                     "hexagonal",
-                                     "cubic",
-                                     "cubic",
-                                     "cubic",
-                                     ],           ## crystal symmetry of material_
-                        "SG": [194,
-                               14,
-                               137,
-                               225,
-                               194,
-                               229,
-                               194,
-                               227,
-                               229,
-                               229
-                               ],                     ## Space group of material_ (None if not known)
-                        "hkl_max_identify" : [5,
-                                              6,
-                                              5,
-                                              5,
-                                              5,
-                                              5,
-                                              5,
-                                              5,
-                                              5,
-                                              5,
-                                              ],        ## Maximum hkl index to classify in a Laue pattern
-                        "nb_grains_per_lp" : [1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              1,
-                                              ],        ## max grains to be generated in a Laue Image
-                        "grains_nb_simulate" : 500,    ## Number of orientations to generate (takes advantage of crystal symmetry)
-                        # =============================================================================
-                        #       Training paarmeters             
-                        # =============================================================================
-                        "freq_rmv_classhkl" : [100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               100,
-                                               ],
-                        "keep_length_classhkl" : [25,
-                                                  100,
-                                                  60,
-                                                  10,
-                                                  25,
-                                                  10,
-                                                  25,
-                                                  10,
-                                                  10,
-                                                  10
-                                                  ],}
+
     ## Import modules used for this Notebook
     import os
     import numpy as np
@@ -304,58 +87,18 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
                     "prefix" : "",
                     "material_": ["Zr_alpha",
                                   "ZrO2_mono",
-                                  "ZrO2_tet",
-                                  "ZrO2_cub",
-                                  "Zr_Nb_Fe",
-                                  "Cr",
-                                  "C14",
-                                  "C15",
-                                  "Zr_beta",
-                                  "Nb_beta"
                                   ],             ## same key as used in dict_LaueTools
                     "symmetry": ["hexagonal",
                                  "monoclinic",
-                                 "tetragonal",
-                                 "cubic",
-                                 "hexagonal",
-                                 "cubic",
-                                 "hexagonal",
-                                 "cubic",
-                                 "cubic",
-                                 "cubic",
                                  ],           ## crystal symmetry of material_
                     "SG": [194,
                            14,
-                           137,
-                           225,
-                           194,
-                           229,
-                           194,
-                           227,
-                           229,
-                           229
                            ],                     ## Space group of material_ (None if not known)
-                    "hkl_max_identify" : [5,
-                                          6,
-                                          5,
-                                          5,
-                                          5,
-                                          5,
-                                          5,
-                                          5,
-                                          5,
-                                          5,
+                    "hkl_max_identify" : [6,
+                                          8,
                                           ],        ## Maximum hkl index to classify in a Laue pattern
-                    "nb_grains_per_lp" : [1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
-                                          1,
+                    "nb_grains_per_lp" : [4,
+                                          4,
                                           ],        ## max grains to be generated in a Laue Image
                     "grains_nb_simulate" : 500,    ## Number of orientations to generate (takes advantage of crystal symmetry)
                     "maximum_angle_to_search":120, ## Angle of radial distribution to reconstruct the histogram (in deg)
@@ -374,32 +117,20 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
                     #       Training paarmeters             
                     # =============================================================================
                     "freq_rmv_classhkl" : [100,
-                                           100,
-                                           100,
-                                           100,
-                                           100,
-                                           100,
-                                           100,
-                                           100,
-                                           100,
                                            100,
                                            ],
-                    "keep_length_classhkl" : [25,
+                    "keep_length_classhkl" : [50,
                                               100,
-                                              60,
-                                              10,
-                                              25,
-                                              10,
-                                              25,
-                                              10,
-                                              10,
-                                              10
                                               ],
+                    "list_hkl_keep" : [
+                                        [(0,0,1)],
+                                        [(0,0,0)]                        
+                                        ],
                     "batch_size":50,               ## batches of files to use while training
-                    "epochs":8,   
+                    "epochs":30,   
                     }
     
-    generate_data = True
+    generate_data = False
     train_model = True
     
     # ### number of files it will generate fro training
@@ -523,10 +254,13 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
         
         freq_rmv = input_params["freq_rmv_classhkl"]
         elements = input_params["keep_length_classhkl"]
+        list_hkl_keep = input_params["list_hkl_keep"]
         
         rmv_freq_class_MM(freq_rmv = freq_rmv, elements = elements,
                           save_directory = save_directory, material_ = material_,
-                          write_to_console = print, progress=None, qapp=None)
+                          write_to_console = print, progress=None, qapp=None,
+                          list_hkl_keep = list_hkl_keep)
+        
         
         ## End of data generation for Neural network training: all files are saved in the same folder 
         ## to be later used for training and prediction
@@ -536,29 +270,26 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
 
     # In[3]:
     if train_model:
+        
         classhkl = np.load(save_directory+"//MOD_grain_classhkl_angbin.npz")["arr_0"]
         angbins = np.load(save_directory+"//MOD_grain_classhkl_angbin.npz")["arr_1"]
         loc_new = np.load(save_directory+"//MOD_grain_classhkl_angbin.npz")["arr_2"]
         with open(save_directory+"//class_weights.pickle", "rb") as input_file:
             class_weights = cPickle.load(input_file)
         class_weights = class_weights[0]
-        
         n_bins = len(angbins)-1
         n_outputs = len(classhkl)
         print(n_bins, n_outputs)
-        
-        
         # ## Step 3: Defining a neural network architecture
         
         # In[4]:
-        
-        
         import tensorflow as tf
         import keras
         from keras.regularizers import l2
         from keras.models import Sequential
         from keras.layers import Dense, Activation, Dropout
-        
+        from tensorflow.keras import Model
+
         metricsNN = [
                     keras.metrics.FalseNegatives(name="fn"),
                     keras.metrics.FalsePositives(name="fp"),
@@ -567,6 +298,62 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
                     keras.metrics.Precision(name="precision"),
                     keras.metrics.Recall(name="accuracy"),
                     ]
+        def model_arch_CNN_DNN_optimized(shape, 
+                                         layer_activation="relu", 
+                                         output_activation="softmax",
+                                         dropout=0.3,
+                                         stride = [1,1],
+                                         kernel_size = [5,5],
+                                         pool_size=[2,2],
+                                         CNN_layers = 2,
+                                         CNN_filters = [32,64],
+                                         DNN_layers = 3,
+                                         DNN_filters = [1000,500,100],
+                                         output_neurons = 11,
+                                         learning_rate = 0.001,
+                                         output="DNN"):            
+            inputs = keras.layers.Input(shape, name="InputLayer")
+            
+            for lay in range(CNN_layers):
+                if lay == 0:
+                    conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
+                                                strides=stride[lay], 
+                                                activation=layer_activation, name="Conv_"+str(lay+1))(inputs)
+                    pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
+                                                      name="Pool_"+str(lay+1))(conv1)
+                else:
+                    conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
+                                                strides=stride[lay], 
+                                                activation=layer_activation, name="Conv_"+str(lay+1))(pool1)
+                    pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
+                                                      name="Pool_"+str(lay+1))(conv1)
+            flatten = keras.layers.Flatten(name="Flatten")(pool1)
+
+            for lay in range(DNN_layers):
+                if lay == 0:
+                    ppKL = keras.layers.Dense(DNN_filters[lay], activation=layer_activation, \
+                                              name="Dense_"+str(lay+1))(flatten)
+                    ppKL = keras.layers.Dropout(dropout, name="Dropout"+str(lay+1))(ppKL)   
+                else:
+                    
+                    ppKL = keras.layers.Dense(DNN_filters[lay], activation=layer_activation, \
+                                              name="Dense_"+str(lay+1))(ppKL)
+                    ppKL = keras.layers.Dropout(dropout, name="Dropout"+str(lay+1))(ppKL) 
+            ## Output layer 
+            if output != "CNN":
+                if DNN_layers == 0:
+                    outputs = keras.layers.Dense(output_neurons, activation=output_activation, name="Dense_out")(flatten)
+                else:
+                    outputs = keras.layers.Dense(output_neurons, activation=output_activation, name="Dense_out")(ppKL)
+            else:
+                outputs = keras.layers.Conv1D(filters=output_neurons, kernel_size=1, 
+                                            strides=1, activation=output_activation, name="Conv_out")(flatten)
+            model = Model(inputs, outputs)
+            ## Compile model
+            otp = tf.keras.optimizers.Adam(learning_rate=learning_rate)
+            model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
+            model.summary()
+            return model
         
         def model_arch_general_optimized(n_bins, n_outputs, kernel_coeff = 0.0005, bias_coeff = 0.0005, lr=None, verbose=1,
                                 write_to_console=None):
@@ -622,7 +409,44 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
                     write_to_console(short_model_summary)
             return model
         
-        
+        def model_arch_general_onelayer(n_bins, n_outputs, 
+                                        kernel_coeff = 0.0005, 
+                                        bias_coeff = 0.0005, lr=None, verbose=1,
+                                        write_to_console=None):
+            """
+            Very simple and straight forward Neural Network with few hyperparameters
+            straighforward RELU activation strategy with cross entropy to identify the HKL
+            Tried BatchNormalization --> no significant impact
+            Tried weighted approach --> not better for HCP
+            Trying Regularaization 
+            l2(0.001) means that every coefficient in the weight matrix of the layer 
+            will add 0.001 * weight_coefficient_value**2 to the total loss of the network
+            1e-3,1e-5,1e-6
+            """
+            model = Sequential()
+            model.add(keras.Input(shape=(n_bins,)))
+            ## Hidden layer 1
+            model.add(Dense(n_bins, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
+            # model.add(BatchNormalization())
+            model.add(Activation('relu'))
+            model.add(Dropout(0.3)) ## Adding dropout as we introduce some uncertain data with noise
+            ## Output layer 
+            model.add(Dense(n_outputs, activation='softmax'))
+            ## Compile model
+            if lr != None:
+                otp = tf.keras.optimizers.Adam(learning_rate=lr)
+                model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
+            else:
+                model.compile(loss='categorical_crossentropy', optimizer="adam", metrics=[metricsNN])
+            
+            if verbose == 1:
+                model.summary()
+                stringlist = []
+                model.summary(print_fn=lambda x: stringlist.append(x))
+                short_model_summary = "\n".join(stringlist)
+                if write_to_console!=None:
+                    write_to_console(short_model_summary)
+            return model
         # ## Step 4: Training  
             
         # In[5]:
@@ -646,10 +470,33 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
         # neurons_multiplier is a list with number of neurons per layer, the first 
         # value is input shape and last value is output shape, 
         # inbetween are the number of neurons per hidden layers
-        model = model_arch_general_optimized(  n_bins, n_outputs,
-                                                kernel_coeff = 1e-5,
-                                                bias_coeff = 1e-6,
-                                                lr = 1e-3,
+        
+        # model = model_arch_general_optimized(  n_bins, n_outputs,
+        #                                         kernel_coeff = 1e-5,
+        #                                         bias_coeff = 1e-6,
+        #                                         lr = 1e-3,
+        #                                         )
+        # model = model_arch_general_onelayer(  n_bins, n_outputs,
+        #                                         kernel_coeff = 1e-5,
+        #                                         bias_coeff = 1e-6,
+        #                                         lr = 1e-3,
+        #                                         )
+        
+        model = model_arch_CNN_DNN_optimized(
+                                                (n_bins, 1), 
+                                                layer_activation="relu", 
+                                                output_activation="softmax",
+                                                dropout=0.2,
+                                                stride = [5,2],
+                                                kernel_size = [10,3],
+                                                pool_size=[2,1],
+                                                CNN_layers = 2,
+                                                CNN_filters = [128,128],
+                                                DNN_layers = 0,
+                                                DNN_filters = [1000,500],
+                                                output_neurons = n_outputs,
+                                                learning_rate = 0.001,
+                                                output="DNN"
                                                 )
         # Save model config and weights
         model_json = model.to_json()
