@@ -58,22 +58,13 @@ if __name__ == "__main__":
                     # =============================================================================
                     "material_": ["Zr_alpha",
                                   "ZrO2_mono",
-                                  "ZrO2_tet",
-                                  "ZrO2_cub",
-                                  "Zr_Nb_Fe",
                                   ],             ## same key as used in dict_LaueTools
                     "prefix" : "",                 ## prefix for the folder to be created for training dataset
                     "symmetry": ["hexagonal",
                                  "monoclinic",
-                                 "tetragonal",
-                                 "cubic",
-                                 "hexagonal",
                                  ],           ## crystal symmetry of material_
                     "SG": [194,
                            14,
-                           137,
-                           225,
-                           194,
                            ],                     ## Space group of material_ (None if not known)
                     # =============================================================================
                     #        Detector parameters (roughly) of the Experimental setup
@@ -88,31 +79,26 @@ if __name__ == "__main__":
                     # =============================================================================
                     #       Prediction paarmeters             
                     # =============================================================================
-                    "experimental_directory": "",
-                    "experimental_prefix": "",
-                    "use_simulated_dataset": True,  ## Use simulated dataset (generated at step 3a) incase no experimental data to verify the trained model
-                    "grid_size_x" : 25,            ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
-                    "grid_size_y" : 25,  
-                    "UB_tolerance": [0.5,
-                                     0.5,
-                                     0.5,
-                                     0.5,
-                                     0.5],
+                    "experimental_directory": r"C:\Users\purushot\Desktop\Guillou_Laue\scan_0012",
+                    "experimental_prefix": "ech1_map2D_3_",
+                    "grid_size_x" : 5,            ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
+                    "grid_size_y" : 5,  
+                    "UB_tolerance": [0.6,
+                                     0.6,
+                                     ],
                     "tolerance_strain": [
-                                        [0.6,0.55,0.5,0.45,0.4,0.35,0.3,0.25,0.2,0.15],
-                                        [0.6,0.55,0.5,0.45,0.4,0.35,0.3,0.25,0.2,0.15],
-                                        [0.6,0.55,0.5,0.45,0.4,0.35,0.3,0.25,0.2,0.15],
                                         [0.6,0.55,0.5,0.45,0.4,0.35,0.3,0.25,0.2,0.15],
                                         [0.6,0.55,0.5,0.45,0.4,0.35,0.3,0.25,0.2,0.15],
                                         ],
                     "strain_free_parameters": ["b","c","alpha","beta","gamma"],
-                    "material_ub_limit": [1000,
-                                          1000,
-                                          1000,
-                                          1000,
-                                          1000],
+                    "material_ub_limit": [10,
+                                          10,
+                                          ],
                     
-                    "UB_matrix_detect": 5
+                    "UB_matrix_detect": 12,
+                    "material_phase_always_present": [1,1,1,1,1,1,1,1,1,2,2,2],
+                    ## in case if one phase is always present in a Laue pattern (useful for substrate cases)
+                    # or for pretty plots
                     }
     
     #%% ## Step 2: Get material parameters 
@@ -132,6 +118,7 @@ if __name__ == "__main__":
     tolerance_strain = input_params["tolerance_strain"]
     strain_free_parameters = input_params["strain_free_parameters"]
     material_limit = input_params["material_ub_limit"]
+    material_phase_always_present = input_params["material_phase_always_present"]
     
     if len(material_) > 1:
         prefix_mat = material_[0]
@@ -171,7 +158,7 @@ if __name__ == "__main__":
         hkl_all_class0.append(hkl_all_class_load)
         
     ## Experimental peak search parameters in case of RAW LAUE PATTERNS from detector
-    intensity_threshold = 50
+    intensity_threshold = 150
     boxsize = 10
     fit_peaks_gaussian = 1
     FitPixelDev = 18
@@ -180,18 +167,17 @@ if __name__ == "__main__":
 
     ## Requirements
     ubmat = input_params["UB_matrix_detect"] # How many orientation matrix to detect per Laue pattern
-    mode_spotCycle = "slow" ## mode of calculation
+    mode_spotCycle = "graphmode" ## mode of calculation
     use_previous_UBmatrix_name = False ## Try previous indexation solutions to speed up the process
     strain_calculation = True ## Strain refinement is required or not
     ccd_label_global = "sCMOS"
 
     ## Parameters to control the orientation matrix indexation
     softmax_threshold_global = 0.80 # softmax_threshold of the Neural network to consider
-    mr_threshold_global = 0.90 # match rate threshold to accept a solution immediately
-    cap_matchrate = 0.01 * 100 ## any UB matrix providing MR less than this will be ignored
+    mr_threshold_global = 0.70 # match rate threshold to accept a solution immediately
+    cap_matchrate = 0.20 * 100 ## any UB matrix providing MR less than this will be ignored
     coeff = 0.10            ## coefficient to calculate the overlap of two solutions
     coeff_overlap = 0.10    ##10% spots overlap is allowed with already indexed orientation
-    material_phase_always_present = "none" ## in case if one phase is always present in a Laue pattern (useful for substrate cases)
     
     ## Additional parameters to refine the orientation matrix construction process
     use_om_user = "false"
@@ -387,48 +373,64 @@ if __name__ == "__main__":
                 crystal,
                 strain_free_parameters] for ii in range(count_global)]
     
+    start_time = time.time()
+    # Launch on single file to verify
     results = new_MP_multimat_function(valu12[0])
-    #%% Launch multiprocessing prediction     
-    # args = zip(valu12)
-    # with multiprocessing.Pool(ncpu) as pool:
-    #     results = pool.starmap(new_MP_multimat_function, tqdm(args, total=len(valu12)))
+    print('Took ',time.time()-start_time, "seconds")
+    
+    print("matching rate")
+    temp_0 = [results[6][i][0][0] for i in range(len(results[6]))]
+    print(temp_0)
+    
+    print("Orientation metrix")
+    temp_0 = [results[2][i][0][0].ravel() for i in range(len(results[6]))]
+    for itemm in temp_0:
+        print(",".join(str(x) for x in itemm))
+    
+    #indexing angles 787
+    
+    #% Launch multiprocessing prediction     
+    if 0:
+        args = zip(valu12)
+        with multiprocessing.Pool(ncpu) as pool:
+            results = pool.starmap(new_MP_multimat_function, tqdm(args, total=len(valu12)))
+            
+            for r in results:
+                r_message_mpdata = r
+                strain_matrix_mpdata, strain_matrixs_mpdata, rotation_matrix_mpdata, col_mpdata,\
+                colx_mpdata, coly_mpdata, match_rate_mpdata, mat_global_mpdata,\
+                    cnt_mpdata, meta_mpdata, files_treated_mpdata, spots_len_mpdata, \
+                        iR_pixel_mpdata, fR_pixel_mpdata, best_match_mpdata, check_mpdata = r_message_mpdata
         
-    #     for r in results:
-    #         r_message_mpdata = r
-    #         strain_matrix_mpdata, strain_matrixs_mpdata, rotation_matrix_mpdata, col_mpdata,\
-    #         colx_mpdata, coly_mpdata, match_rate_mpdata, mat_global_mpdata,\
-    #             cnt_mpdata, meta_mpdata, files_treated_mpdata, spots_len_mpdata, \
-    #                 iR_pixel_mpdata, fR_pixel_mpdata, best_match_mpdata, check_mpdata = r_message_mpdata
-    
-    #         for i_mpdata in files_treated_mpdata:
-    #             files_treated.append(i_mpdata)
-    
-    #         for intmat_mpdata in range(int(ubmat)):
-    #             check[cnt_mpdata,intmat_mpdata] = check_mpdata[cnt_mpdata,intmat_mpdata]
-    #             mat_global[intmat_mpdata][0][cnt_mpdata] = mat_global_mpdata[intmat_mpdata][0][cnt_mpdata]
-    #             strain_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-    #             strain_matrixs[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrixs_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-    #             rotation_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = rotation_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-    #             col[intmat_mpdata][0][cnt_mpdata,:] = col_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-    #             colx[intmat_mpdata][0][cnt_mpdata,:] = colx_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-    #             coly[intmat_mpdata][0][cnt_mpdata,:] = coly_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-    #             match_rate[intmat_mpdata][0][cnt_mpdata] = match_rate_mpdata[intmat_mpdata][0][cnt_mpdata]
-    #             spots_len[intmat_mpdata][0][cnt_mpdata] = spots_len_mpdata[intmat_mpdata][0][cnt_mpdata]
-    #             iR_pix[intmat_mpdata][0][cnt_mpdata] = iR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
-    #             fR_pix[intmat_mpdata][0][cnt_mpdata] = fR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
-    #             best_match[intmat_mpdata][0][cnt_mpdata] = best_match_mpdata[intmat_mpdata][0][cnt_mpdata]
+                for i_mpdata in files_treated_mpdata:
+                    files_treated.append(i_mpdata)
+        
+                for intmat_mpdata in range(int(ubmat)):
+                    check[cnt_mpdata,intmat_mpdata] = check_mpdata[cnt_mpdata,intmat_mpdata]
+                    mat_global[intmat_mpdata][0][cnt_mpdata] = mat_global_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    strain_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    strain_matrixs[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrixs_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    rotation_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = rotation_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    col[intmat_mpdata][0][cnt_mpdata,:] = col_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    colx[intmat_mpdata][0][cnt_mpdata,:] = colx_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    coly[intmat_mpdata][0][cnt_mpdata,:] = coly_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    match_rate[intmat_mpdata][0][cnt_mpdata] = match_rate_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    spots_len[intmat_mpdata][0][cnt_mpdata] = spots_len_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    iR_pix[intmat_mpdata][0][cnt_mpdata] = iR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    fR_pix[intmat_mpdata][0][cnt_mpdata] = fR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    best_match[intmat_mpdata][0][cnt_mpdata] = best_match_mpdata[intmat_mpdata][0][cnt_mpdata]
                 
-    # #%% Save results
-    # save_directory_ = filenameDirec+"//results_"+prefix_mat+"_"+c_time
-    # if not os.path.exists(save_directory_):
-    #     os.makedirs(save_directory_)
-        
-    # ## intermediate saving of pickle objects with results
-    # with open(save_directory_+ "//results.pickle", "wb") as output_file:
-    #         cPickle.dump([best_match, mat_global, rotation_matrix, strain_matrix, 
-    #                       strain_matrixs, col, colx, coly, match_rate, files_treated,
-    #                       lim_x, lim_y, spots_len, iR_pix, fR_pix,
-    #                       material_, lattice_material,
-    #                       symmetry, crystal], output_file)
-    # print("data saved in ", save_directory_)
+        #%% Save results
+        save_directory_ = filenameDirec+"//results_"+prefix_mat+"_"+c_time
+        if not os.path.exists(save_directory_):
+            os.makedirs(save_directory_)
+            
+        ## intermediate saving of pickle objects with results
+        with open(save_directory_+ "//results.pickle", "wb") as output_file:
+                cPickle.dump([best_match, mat_global, rotation_matrix, strain_matrix, 
+                              strain_matrixs, col, colx, coly, match_rate, files_treated,
+                              lim_x, lim_y, spots_len, iR_pix, fR_pix,
+                              material_, lattice_material,
+                              symmetry, crystal], output_file)
+        print("data saved in ", save_directory_)
 
